# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2019. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

import os
from _snap_ml.utils import _is_mpi_enabled, _is_data_snapml_partition_type
from scipy.sparse import issparse

_mpi_enabled = _is_mpi_enabled()

# Custom decorator to capture user passed arguments
def _check_uparams(fun):
    fvars = list(fun.__code__.co_varnames)
    def filterFn(x):
        filterItems = ['self']
        if x in filterItems:
            return False
        else:
            return True

    def wrapFunc(self, **kwargs):
        self._uparams = set(list(kwargs.keys()))
        self._param_list = list(filter(filterFn, fvars))
        return fun(self, **kwargs)
    return wrapFunc

def _is_gpu_matrix_type(X,y):
    if (type(X).__name__ == "DeviceNDArray" and 
            type(y).__name__ == "DeviceNDArray"):
        return True
    return False

def _check_snapml_types(X, y):
    # SnapML can be used only for these types
    if _check_snapml_type(X) and _check_snapml_type(y):
        return True
    return False

def _check_snapml_type(X):
    # SnapML can be used only for these types
    if _is_data_snapml_partition_type(X):
        return True

    if (type(X).__name__ == "csr_matrix" or
        type(X).__name__ == "csc_matrix" or
        type(X).__name__ == "ndarray" or
        type(X).__name__ == "DeviceNDArray"):
        return True

    return False

# Return data size for a given array
def _get_data_size(X):
    # Can't determine size for DeviceNDArray
    if (type(X).__name__ == "DeviceNDArray"):
        return -1

    if issparse(X):
        # return int((X.data.nbytes + X.indptr.nbytes + X.indices.nbytes)/1024)
        return int((X.data.nbytes)/1024)
    elif type(X).__name__ == "ndarray" :
        return int(X.nbytes/1024)
    else:
        return -1

